package net.kronos.orm.rowset;


import java.lang.reflect.Field;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import net.kronos.orm.factory.Column;

import com.sun.rowset.CachedRowSetImpl;


public class ObjectCachedRowSet extends CachedRowSetImpl {
	private static final long serialVersionUID = 1L;
	
	public ObjectCachedRowSet() throws SQLException{
		super();
	}

	/**
	 * <pre>
	 * カーソルのあたっている1行のデータを引数クラスのインスタンスに詰め込んで返します。
	 * 詰め込む際に以下のルールに従います。
	 * 1.アノテーションにしたがって詰め込みます
	 * 2.フィールド名とカラム名が一致する場合に詰め込みます。
	 * 3.アンダースコアを除去してチェックします。
	 * </pre>
	 * @param <T>
	 * @param clazz 戻り値を表すクラス
	 * @return 引数クラスのインスタンス
	 * @throws SQLException
	 */
	public<T> T getEntity(Class<T> clazz) throws SQLException{
		try {
			boolean isMatch = false;
			Object o = clazz.newInstance();
			Field[] fields = clazz.getDeclaredFields();
			ResultSetMetaData rsmd =  this.getMetaData();
			
			for(Field f : fields){
				for (int colinx = 1;colinx <= rsmd.getColumnCount();colinx++){
					String  colName = rsmd.getColumnName(colinx);
					isMatch = isMatch(f,colName);
					if(isMatch){
						o = mappingColumnToField(rsmd.getColumnName(colinx),f,o);
						break;
					}
				}
			}
			return clazz.cast(o);
		} catch (InstantiationException e) {
			throw ExceptionHandler.wrapNoCheckException(e);
		} catch (IllegalAccessException e) {
			throw ExceptionHandler.wrapNoCheckException(e);
		} catch (SecurityException e) {
			throw ExceptionHandler.wrapNoCheckException(e);
		} 
	}

	/**
	 * カラム名を変換してフィールド名と一致するかチェックします。
	 * 全て大文字小文字は問いません。
	 * 1.Columnアノテーションをチェック
	 * 2.アンダースコア(_)を除去してチェック
	 * @param field フィールドリフレクション
	 * @param colName カラム名
	 * @return true:マッチする
	 */
	private boolean isMatch(Field field ,String colName){
		boolean isMatch = false;
		String lowerColName  = colName.toLowerCase();
		
		Column annotation = field.getAnnotation(Column.class); 
		if(annotation != null && lowerColName.equals(annotation.value().toLowerCase())){
			isMatch = true;
		}
		
		String lowerFieldName = field.getName().toLowerCase();
		if(lowerColName.replaceAll("_", "").equals(lowerFieldName)){
			isMatch = true;
		}
		return isMatch;
	}
	
	/**
	 * カラムから取得したデータを対象フィールドにセットします。
	 * @param columnName カラム名
	 * @param field　フィールド名
	 * @param target 対象インスタンス
	 * @return 詰め込み済みインスタンス
	 */
	private Object mappingColumnToField(String columnName,Field field,Object target){
		boolean isFieldLockReleased = false;
		// フィールドのアクセスを一時的に許す
		if(!field.isAccessible()){
			field.setAccessible(true);
			isFieldLockReleased = true;
		}
		
		// カラムからデータを取得しフィールドにセット
		Object data;
		try {
			data = this.getObject(columnName);
			field.set(target, data);
		} catch (SQLException e) {
			throw ExceptionHandler.wrapNoCheckException(e);
		} catch (IllegalArgumentException e) {
			throw ExceptionHandler.wrapNoCheckException(e);
		} catch (IllegalAccessException e) {
			throw ExceptionHandler.wrapNoCheckException(e);
		}
		
		// フィールドのアクセス権を元に戻す。
		if(isFieldLockReleased){
			field.setAccessible(true);
		}
		return target;
	}
	

}
